@@ -2,7 +2,13 @@ import re
 import urllib.parse
 
 from .common import InfoExtractor
-from ..utils import str_or_none, traverse_obj
+from ..networking import HEADRequest
+from ..utils import (
+    determine_ext,
+    js_to_json,
+    str_or_none,
+)
+from ..utils.traversal import traverse_obj
 
 
 class SubstackIE(InfoExtractor):
@@ -14,11 +20,11 @@ class SubstackIE(InfoExtractor):
             'id': '47660949',
             'ext': 'mp4',
             'title': 'I MADE A VLOG',
-            'description': 'md5:10c01ff93439a62e70ce963b2aa0b7f6',
+            'description': 'md5:9248af9a759321e1027226f988f54d96',
             'thumbnail': 'md5:bec758a34d8ee9142d43bcebdf33af18',
             'uploader': 'Maybe Baby',
             'uploader_id': '33628',
-        }
+        },
     }, {
         'url': 'https://haleynahman.substack.com/p/-dear-danny-i-found-my-boyfriends?s=r',
         'md5': '0a63eacec877a1171a62cfa69710fcea',
@@ -30,7 +36,7 @@ class SubstackIE(InfoExtractor):
             'thumbnail': 'md5:daa40b6b79249417c14ff8103db29639',
             'uploader': 'Maybe Baby',
             'uploader_id': '33628',
-        }
+        },
     }, {
         'url': 'https://andrewzimmern.substack.com/p/mussels-with-black-bean-sauce-recipe',
         'md5': 'fd3c07077b02444ff0130715b5f632bb',
@@ -42,7 +48,43 @@ class SubstackIE(InfoExtractor):
             'thumbnail': 'md5:e30bfaa9da40e82aa62354263a9dd232',
             'uploader': "Andrew Zimmern's Spilled Milk ",
             'uploader_id': '577659',
-        }
+        },
+    }, {
+        # Podcast that needs its file extension resolved to mp3
+        'url': 'https://persuasion1.substack.com/p/summers',
+        'md5': '1456a755d46084744facdfac9edf900f',
+        'info_dict': {
+            'id': '141970405',
+            'ext': 'mp3',
+            'title': 'Larry Summers on What Went Wrong on Campus',
+            'description': 'Yascha Mounk and Larry Summers also discuss the promise and perils of artificial intelligence.',
+            'thumbnail': r're:https://substackcdn\.com/image/.+\.jpeg',
+            'uploader': 'Persuasion',
+            'uploader_id': '61579',
+        },
+    }]
+    _WEBPAGE_TESTS = [{
+        'url': 'https://www.mollymovieclub.com/p/interstellar',
+        'info_dict': {
+            'id': '53602801',
+            'ext': 'mpga',
+            'title': 'Interstellar',
+            'description': 'Listen now | Episode One',
+            'thumbnail': r're:https?://.+\.jpeg',
+            'uploader': 'Molly Movie Club',
+            'uploader_id': '839621',
+        },
+    }, {
+        'url': 'https://www.blockedandreported.org/p/episode-117-lets-talk-about-depp',
+        'info_dict': {
+            'id': '57962052',
+            'ext': 'mpga',
+            'title': 'md5:855b2756f0ee10f6723fa00b16266f8d',
+            'description': 'The takes the takes the takes',
+            'thumbnail': r're:https?://.+\.jpeg',
+            'uploader': 'Blocked and Reported',
+            'uploader_id': '500230',
+        },
     }]
 
     @classmethod
@@ -50,16 +92,16 @@ class SubstackIE(InfoExtractor):
         if not re.search(r'<script[^>]+src=["\']https://substackcdn.com/[^"\']+\.js', webpage):
             return
 
-        mobj = re.search(r'{[^}]*["\']subdomain["\']\s*:\s*["\'](?P<subdomain>[^"]+)', webpage)
+        mobj = re.search(r'{[^}]*\\?["\']subdomain\\?["\']\s*:\s*\\?["\'](?P<subdomain>[^\\"\']+)', webpage)
         if mobj:
             parsed = urllib.parse.urlparse(url)
             yield parsed._replace(netloc=f'{mobj.group("subdomain")}.substack.com').geturl()
-            raise cls.StopExtraction()
+            raise cls.StopExtraction
 
-    def _extract_video_formats(self, video_id, username):
+    def _extract_video_formats(self, video_id, url):
         formats, subtitles = [], {}
         for video_format in ('hls', 'mp4'):
-            video_url = f'https://{username}.substack.com/api/v1/video/upload/{video_id}/src?type={video_format}'
+            video_url = urllib.parse.urljoin(url, f'/api/v1/video/upload/{video_id}/src?type={video_format}')
 
             if video_format == 'hls':
                 fmts, subs = self._extract_m3u8_formats_and_subtitles(video_url, video_id, 'mp4', fatal=False)
@@ -77,14 +119,29 @@ class SubstackIE(InfoExtractor):
         display_id, username = self._match_valid_url(url).group('id', 'username')
         webpage = self._download_webpage(url, display_id)
 
-        webpage_info = self._search_json(r'<script[^>]*>\s*window\._preloads\s*=', webpage, 'preloads', display_id)
+        webpage_info = self._parse_json(self._search_json(
+            r'window\._preloads\s*=\s*JSON\.parse\(', webpage, 'json string',
+            display_id, transform_source=js_to_json, contains_pattern=r'"{(?s:.+)}"'), display_id)
+
+        canonical_url = url
+        domain = traverse_obj(webpage_info, ('domainInfo', 'customDomain', {str}))
+        if domain:
+            canonical_url = urllib.parse.urlparse(url)._replace(netloc=domain).geturl()
 
         post_type = webpage_info['post']['type']
         formats, subtitles = [], {}
         if post_type == 'podcast':
-            formats, subtitles = [{'url': webpage_info['post']['podcast_url']}], {}
+            fmt = {'url': webpage_info['post']['podcast_url']}
+            if not determine_ext(fmt['url'], default_ext=None):
+                # The redirected format URL expires but the original URL doesn't,
+                # so we only want to extract the extension from this request
+                fmt['ext'] = determine_ext(self._request_webpage(
+                    HEADRequest(fmt['url']), display_id,
+                    'Resolving podcast file extension',
+                    'Podcast URL is invalid').url)
+            formats.append(fmt)
         elif post_type == 'video':
-            formats, subtitles = self._extract_video_formats(webpage_info['post']['videoUpload']['id'], username)
+            formats, subtitles = self._extract_video_formats(webpage_info['post']['videoUpload']['id'], canonical_url)
         else:
             self.raise_no_formats(f'Page type "{post_type}" is not supported')
 
@@ -97,4 +154,5 @@ class SubstackIE(InfoExtractor):
             'thumbnail': traverse_obj(webpage_info, ('post', 'cover_image')),
             'uploader': traverse_obj(webpage_info, ('pub', 'name')),
             'uploader_id': str_or_none(traverse_obj(webpage_info, ('post', 'publication_id'))),
+            'webpage_url': canonical_url,
         }
